@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2021 Red Hat, Inc.  All rights reserved.
+ * Copyright (C) 2017-2024 Red Hat, Inc.  All rights reserved.
  *
  * Author: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *
@@ -606,7 +606,7 @@ static int opensslcrypto_init(
 #endif
 
 	log_debug(knet_h, KNET_SUB_OPENSSLCRYPTO,
-		  "Initizializing openssl crypto module [%s/%s]",
+		  "Initializing openssl crypto module [%s/%s]",
 		  knet_handle_crypto_cfg->crypto_cipher_type,
 		  knet_handle_crypto_cfg->crypto_hash_type);
 
