@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016-2023 Red Hat, Inc.  All rights reserved.
+ * Copyright (C) 2016-2024 Red Hat, Inc.  All rights reserved.
  *
  * Authors: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *
@@ -130,10 +130,10 @@ static void parse_nodes(char *nodesinfo[MAX_NODES], int onidx, int port, struct
 {
 	int i;
 	char *temp = NULL;
-	char port_str[10];
+	char port_str[11];
 
 	memset(port_str, 0, sizeof(port_str));
-	sprintf(port_str, "%d", port);
+	snprintf(port_str, sizeof(port_str), "%d", port);
 
 	for (i = 0; i < onidx; i++) {
 		nodes[i].nodeid = atoi(strtok(nodesinfo[i], ","));
