@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2021 Red Hat, Inc.  All rights reserved.
+ * Copyright (C) 2017-2024 Red Hat, Inc.  All rights reserved.
  *
  * Authors: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *
@@ -28,7 +28,7 @@ int sctp_transport_link_clear_config(knet_handle_t knet_h, struct knet_link *kn_
 int sctp_transport_free(knet_handle_t knet_h);
 int sctp_transport_init(knet_handle_t knet_h);
 int sctp_transport_rx_sock_error(knet_handle_t knet_h, int sockfd, int recv_err, int recv_errno);
-int sctp_transport_tx_sock_error(knet_handle_t knet_h, int sockfd, int recv_err, int recv_errno);
+int sctp_transport_tx_sock_error(knet_handle_t knet_h, int sockfd, int subsys, int recv_err, int recv_errno);
 int sctp_transport_rx_is_data(knet_handle_t knet_h, int sockfd, struct knet_mmsghdr *msg);
 int sctp_transport_link_dyn_connect(knet_handle_t knet_h, int sockfd, struct knet_link *kn_link);
 int sctp_transport_link_is_down(knet_handle_t knet_h, struct knet_link *kn_link);
